@@ -68,7 +68,7 @@ extern	void	init_control	(void);
 extern	void	process_control (struct recvbuf *, int);
 extern	void	report_event	(int, struct peer *, const char *);
 extern	int	mprintf_event	(int, struct peer *, const char *, ...)
-			__attribute__((__format__(__printf__, 3, 4)));
+			NTP_PRINTF(3, 4);
 
 /* ntp_control.c */
 /*
@@ -78,7 +78,7 @@ extern	int	mprintf_event	(int, struct peer *, const char *, ...)
 struct ctl_var {
 	u_short code;
 	u_short flags;
-	char *text;
+	const char *text;
 };
 /*
  * Flag values
@@ -117,7 +117,9 @@ extern	endpt *	findbcastinter		(sockaddr_u *);
 extern	void	enable_broadcast	(endpt *, sockaddr_u *);
 extern	void	enable_multicast_if	(endpt *, sockaddr_u *);
 extern	void	interface_update	(interface_receiver_t, void *);
-
+#ifndef HAVE_IO_COMPLETION_PORT
+extern  void    io_handler              (void);
+#endif
 extern	void	init_io 	(void);
 extern	void	io_open_sockets	(void);
 extern	void	io_clr_stats	(void);
@@ -216,15 +218,12 @@ extern struct value tai_leap;
 /* ntp_proto.c */
 extern	void	transmit	(struct peer *);
 extern	void	receive 	(struct recvbuf *);
-extern	void	peer_clear	(struct peer *, char *);
+extern	void	peer_clear	(struct peer *, const char *);
 extern	void 	process_packet	(struct peer *, struct pkt *, u_int);
 extern	void	clock_select	(void);
 
-extern	int	leap_tai;	/* TAI at next leap */
-extern	u_long	leap_sec;	/* next scheduled leap from file */
-extern	u_long	leap_peers;	/* next scheduled leap from peers */
-extern	u_long	leapsec;	/* seconds to next leap */
-extern	u_long	leap_expire;	/* leap information expiration */
+extern	u_long	leapsec;	/* seconds to next leap (proximity class) */
+extern  int     leapdif;        /* TAI difference step at next leap second*/
 extern	int	sys_orphan;
 extern	double	sys_mindisp;
 extern	double	sys_maxdist;
@@ -285,10 +284,9 @@ extern	void	record_proto_stats (char *);
 extern	void	record_loop_stats (double, double, double, double, int);
 extern	void	record_clock_stats (sockaddr_u *, const char *);
 extern	int	mprintf_clock_stats(sockaddr_u *, const char *, ...)
-			__attribute__((__format__(__printf__, 2, 3)));
-extern	void	record_raw_stats (sockaddr_u *srcadr, sockaddr_u *dstadr, l_fp *t1, l_fp *t2, l_fp *t3, l_fp *t4, int leap, int version, int mode, int stratum, int poll, int precision, double root_delay, double root_dispersion, u_int32 refid);
-extern	void	check_leap_file	(void);
-extern	u_long	leap_month(u_long);
+			NTP_PRINTF(2, 3);
+extern	void	record_raw_stats (sockaddr_u *srcadr, sockaddr_u *dstadr, l_fp *t1, l_fp *t2, l_fp *t3, l_fp *t4, int leap, int version, int mode, int stratum, int ppoll, int precision, double root_delay, double root_dispersion, u_int32 refid);
+extern	void	check_leap_file	(int is_daily_check, u_int32 ntptime, const time_t * systime);
 extern	void	record_crypto_stats (sockaddr_u *, const char *);
 #ifdef DEBUG
 extern	void	record_timing_stats (const char *);
@@ -378,8 +376,6 @@ extern u_long	io_timereset;		/* time counters were reset */
 
 /* ntp_io.c */
 extern  int	disable_dynamic_updates;
-extern fd_set	activefds;
-extern int	maxactivefd;
 extern u_int	sys_ifnum;		/* next .ifnum to assign */
 extern endpt *	any_interface;		/* IPv4 wildcard */
 extern endpt *	any6_interface;		/* IPv6 wildcard */
@@ -389,7 +385,8 @@ extern endpt *	ep_list;		/* linked list */
 /* ntp_loopfilter.c */
 extern double	drift_comp;		/* clock frequency (s/s) */
 extern double	clock_stability;	/* clock stability (s/s) */
-extern double	clock_max;		/* max offset before step (s) */
+extern double	clock_max_back;		/* max backward offset before step (s) */
+extern double	clock_max_fwd;		/* max forward offset before step (s) */
 extern double	clock_panic;		/* max offset before panic (s) */
 extern double	clock_phi;		/* dispersion rate (s/s) */
 extern double	clock_minstep;		/* step timeout (s) */
@@ -404,7 +401,7 @@ extern int	pll_status;		/* status bits for kernel pll */
 extern int	ntp_enable;		/* clock discipline enabled */
 extern int	pll_control;		/* kernel support available */
 extern int	kern_enable;		/* kernel support enabled */
-extern int	pps_enable;		/* kernel PPS discipline enabled */
+extern int	hardpps_enable;		/* kernel PPS discipline enabled */
 extern int	ext_enable;		/* external clock enabled */
 extern int	cal_enable;		/* refclock calibrate enable */
 extern int	allow_panic;		/* allow panic correction */
